#pragma rtGlobals=1		// Use modern global access method.
#pragma version=1.0

// REVISION HISTORY
// Mar. 29, 2007		ACL			Initial Release

//	ACL_WordWrapControlText.ipf
//	
//	This function returns a string that adds line breaks (\r) to a string of text such
//	that when the text is displayed in a TitleBox control on a panel, the text will fit 
//	within the boundaries of a GroupBox control on the same panel.  The TitleBox
//	and GroupBox do not have to be overlapping, but in most cases it makes sense
//	to have the upper left corner of the TitleBox placed in the upper left corner
//	of the GroupBox.  The GroupBox control does not need to be visible--in other 
//	words, you can set its disable state to hidden (disable=1).
//	
//	This function calculates the length of the text based upon the font name,
//	size, and style of the TitleBox control.  If any of those parameters are not
//	specified, the function will use the default values for the given window.
//	
//	The TitleBox and GroupBox can be on a stand alone panel window 
//	or on a control bar of a graph.
//	
//	There are a few cases in which this function will fail to return a text string
//	that truly 'fits' within the given GroupBox:
//	1.	Any word that is wider than the GroupBox itself will fall partially outside
//		of the GroupBox
//	2.	This function includes the length of Igor control characters/text with the 
//		width of a string, so use of any control characters will throw off the
//		appearance of the wrapping (in other words, line breaks will be
//		early or late, depending on what kind of formatting is represented
//		by the control characters/text).
//		
//	For additional information, please see the ACL_WordWrapControlText.ihf help file or the
//	ACL_WordWrapControlTextDemo.pxp experiment.



//**
// Adds line breaks (\r) to a string of text contained in a TitleBox control
// so that the text will fit within the boundaries of the specified GroupBox control.
//
// @param textStr
// 	String to be word wrapped.  NOTE:  Do not use the same string variable
// 	that is displayed in the TitleBox control here, or otherwise the results may not
// 	be what you expect.
// @param outputTextWave
// 	A text wave where the wrapped output will also be stored
// 	each line will be placed into a row of the text wave.
// 	The wave will contain the entire parsed text string, and NOT just
// 	the portion of the text string that is displayed in the TitleBox control.
// @param panelNameStr
// 	Name of the panel on which ctrlNameStr resides.
// @param ctrlNameStr
// 	Name of the TitleBox control that contains the text that will be displayed.
// @param groupNameStr
// 	Name of a GroupBox control
// 	that can be used to outline the text and that this function
// 	uses to determine the dimensions over which the text 
// 	will be placed.  This must be on the same window as the text control.
// 	If you don't want the group box to be visible on the panel, you can
// 	set the state of the group box to hidden (disable=1).
// @param firstLineNum
// 	OPTIONAL:  If the string is too long to fit in the GroupBox height-wise
// 	the string will be trimmed so that the beginning of the string starts
// 	with the text that makes up firstLineNum.  Default value is 0, which
// 	means that the returned string will start from the beginning.  This line
// 	number is zero based.
// 	This parameter can also be used in conjunction with a SetVariable or Slider
// 	control to provide similar functionality to a scroll bar in a standard Windows
// 	or Mac text window.
//
// @return
// 	textStr with line breaks (\r) inserted in the appropriate places.
//*
Function/S WordWrapControlText(textStr, outputTextWave, panelNameStr, ctrlNameStr, groupNameStr, [firstLineNum])

	String textStr
	WAVE/T outputTextWave
	String panelNameStr
	String ctrlNameStr
	String groupNameStr
	Variable firstLineNum
							
	String returnTextStr = ""
	
	if (ParamIsDefault(firstLineNum))
		firstLineNum = 0
	elseif (firstLineNum < 0)
		firstLineNum = 0
	endif
	
	Variable currentLineNum = 0
	Redimension/N=(0) outputTextWave
	// determine the dimensions of the control where this text will be placed
	Variable textWidth, textHeight	// Width and Height are in pixels, font is in points
	
	ControlInfo/W=$(panelNameStr) $(groupNameStr)
	if (abs(V_Flag) == 9)		// group box control
		textWidth = V_Width
		textHeight = V_Height
	else						// control doesn't exist	
		return textStr				
	endif

	// get the font information for the default font used in this panel
	String defaultFontName
	Variable defaultTextFontSize, defaultTextFontStyle
	DefaultGuiFont/W=$(panelNameStr) all
	defaultFontName = S_name
	defaultTextFontSize = V_value
	defaultTextFontStyle = V_flag

	// now read information from title control to see if
	// a non-default font size, style, or name is being used	
	Variable textFontSize, textFontStyle
	String fontName, escapedFontName
	ControlInfo/W=$(panelNameStr) $(ctrlNameStr)
	if (abs(V_Flag) == 10)		// title box control
		textFontSize = str2num(StringByKey("fSize", S_recreation, "=", ","))
		if (numtype(textFontSize) != 0)		// default font size is used
			textFontSize = defaultTextFontSize
			if (numtype(textFontSize) != 0)
				textFontSize = 12
			endif
		endif
		textFontStyle = str2num(StringByKey("fStyle", S_recreation, "=", ","))
		if (numtype(textFontStyle) != 0)		// default font style is used
			textFontStyle = defaultTextFontStyle
			if (numtype(textFontStyle) != 0)
				textFontStyle = 0
			endif
		endif		
		escapedFontName = StringByKey("font", S_recreation, "=", ",")
		// The recreation macro will now store font with escaped quotes, for example
		// font=\"Arial\"
		// So we need to get rid of the \" part, if it's present.
		// The regex below will match the following values of escapedFontName
		// \"Arial\"
		// Arial
		// and will also match "".
		SplitString /E="(?:\\\")?([\w _+-]*)(?:\\\")*" escapedFontName, fontName
		if (V_Flag == 1)
			if (cmpstr(fontName, "") == 0)	// no font name found
				fontName = defaultFontName
				if (cmpstr(fontName, "") == 0)		// will be true if S_name above is ""
					fontName = GetDefaultFont(panelNameStr)
				endif
			endif
		endif
	endif
	
	// Determine the height, in pixels, of a line of text
	Variable lineHeight = FontSizeHeight(fontName, textFontSize, textFontStyle)
	Variable maxNumLines = floor(textHeight / lineHeight)
	if (maxNumLines == 0)
		return ""		// GroupBox is too short for even one line of text
	endif

	
	// search for spaces, check length of string up to that point + length of new text, and decide whether 
	// to add the new word or add a line break
	Variable sourceStringPos = 0
	Variable nextSpacePos, nextCRPos, nextBreakPos
	Variable currentTextWidth, newTextWidth
	Variable breakIsCR = 0
	String nextWordString = ""
	String currentLineString = ""

	do
		nextSpacePos = strsearch(textStr, " ", sourceStringPos)
		nextSpacePos = nextSpacePos >= 0 ? nextSpacePos : inf		// set to inf if space is not found
		nextCRPos = strsearch(textStr, "\r", sourceStringPos)
		nextCRPos = nextCRPos >= 0 ? nextCRPos : inf		// set to inf if \r is not found
		if (nextCRPos >= 0 && nextCRPos < nextSpacePos)
			breakIsCR = 1
		else
			breakIsCR = 0
		endif
		nextBreakPos = min(nextSpacePos, nextCRPos)

		if (numtype(nextBreakPos) == 1)		// nextBreakPos == inf means there are no more spaces or \r
			if (strlen(textStr) == sourceStringPos)	// at the end of the string
				returnTextStr += currentLineString	
				break
			else
				nextWordString = textStr[sourceStringPos, inf]
				sourceStringPos = strlen(textStr)
			endif
		else
			nextWordString = textStr[sourceStringPos, nextBreakPos]
		endif
		currentTextWidth = FontSizeStringWidth(fontName, textFontSize, textFontStyle, currentLineString)
		newTextWidth = FontSizeStringWidth(fontName, textFontSize, textFontStyle, nextWordString)
		if ((currentTextWidth + newTextWidth + 5) <= textWidth)		// add this word		// leave 5px padding
			currentLineString += nextWordString
			if (numtype(nextBreakPos) == 1)
				break
			elseif (breakIsCR)
				sourceStringPos = nextBreakPos + 1
				returnTextStr += currentLineString
				Redimension/N=(currentLineNum + 1) outputTextWave
				outputTextWave[currentLineNum] = currentLineString
				currentLineString = ""
				currentLineNum += 1
			else
				sourceStringPos = nextBreakPos + 1
			endif
		else													// add a new line and then add this word
			returnTextStr += currentLineString
			returnTextStr += "\r"
			Redimension/N=(currentLineNum + 1) outputTextWave
			outputTextWave[currentLineNum] = currentLineString + "\r"
			currentLineNum += 1
			currentLineString = nextWordString
			if (numtype(nextBreakPos) == 1)
				break
			else
				sourceStringPos = nextBreakPos + 1
			endif
		endif
	while(sourceStringPos < strlen(textStr) - 1)
	returnTextStr += currentLineString		// add last part of string to return string
	Redimension/N=(currentLineNum + 1) outputTextWave
	outputTextWave[currentLineNum] = currentLineString + "\r"
	
	Variable n
	String finalOutputString = ""
	Variable numLinesToReturn = min(DimSize(outputTextWave, 0), maxNumLines + firstLineNum)
	For (n=firstLineNum; n<numLinesToReturn; n+=1)
		finalOutputString += outputTextWave[n]
	EndFor

	return finalOutputString
End